#include "Log.h"
#include "stdarg.h"
#include "string.h"
#include "stdio.h"
#include "time.h"
#include "Elipsis.h"
#include "SysAssert.h"
#include "Lock.h"
#include "FileReport.h"

#define WORKSTRING_LEN    (1024*64)
#define MAXTAB_LEN        (80)   // Should be less that WORKSTRING_LEN

FileReportOpenFn  *fileReportOpen  = 0;
FileReportCloseFn *fileReportClose = 0;
FileReportKillFn  *fileReportKill  = 0;
FileReportMakeFn  *fileReportMake  = 0;

int    logLock = 0;
char * logTell  = "*Tell";
char * logFatal = "!Fatal";

LogCallbackFn *  logCallback     = NULL;
FILE *           logStream       = NULL;
int              logIndent       = 0;
int              logFlushAlways  = 0;
int              logSuperCareful = 0;
int              logToStdErr     = 1;
char             logFileName[256] = {0,};
int              logDoNotShow    = 0;

struct LogDestructor
{
	~LogDestructor()
	{
		if( logStream )
		{
			if( fileReportClose )
				fileReportClose( (int)logStream, logFileName );
			fclose( logStream );
			logStream = 0;
		}
	}
};
LogDestructor logDestructor;

// typedef void LogCallbackFn( char *groupName, char *s );

void logSetCallbackFn( LogCallbackFn *callback )
{
	logCallback = callback;
}

void logSetIndent( int indent ) 
{
	logIndent = indent;
}

void logTabMore() 
{
	// Not using min max to simplify UNIX compatibility
	logIndent += 2;
	if( logIndent > MAXTAB_LEN )
	{
		logIndent = MAXTAB_LEN;
	}
}

void logTabLess() 
{
	logIndent -= 2;
	if( logIndent < 0 )
	{
		logIndent = 0;
	}
}

void logFlush()
{
	if( logStream != NULL )
	{
		fflush( logStream );
	}
}

void logSetFlushAlways( int newValue )
{
	logFlushAlways = newValue;
}

void logClose()
{
	if( logStream )
	{
		sysAssert( logStream );
		if( fileReportClose )
			fileReportClose( (int)logStream, logFileName );
		
		// This is just in case the fileReprtClose interacts with
		// the log.
		if( logStream )
		{
			fclose( logStream );
		}
	}
	logStream = NULL;
}

int logOpenFile( char *fileName, char *mode )
{
	if( logDoNotShow )
		return 1;

	int ok = 1;
	sysAssert( fileName && *fileName );

	if( !strnicmp( "file:///", fileName, 8 ) )
		strcpy( logFileName, fileName+8 );
	else
		strcpy( logFileName, fileName );

	logStream = fopen( logFileName, mode );
	if( fileReportOpen )
		fileReportOpen( (int)logStream, logFileName, 0, 1, 1 );
	ok = (logStream != 0 );
	if( logSuperCareful )
	{
		logClose();
	}
	return ok;
}

int logOpen( char *fileName )
{
	return logOpenFile( fileName, "w+b" );
}

void logClear()
{
	logClose();
	unlink( logFileName );
}


int logAppend( char *fileName )
{
	return logOpenFile( fileName, "a+b" );
}


void logSetCloseAlways( int newValue )
{
	logSuperCareful = newValue;
	logClose();
}

void logAdd( char *groupName, char *fmt, ... )
{
	if( logDoNotShow )
		return;

	char workString[WORKSTRING_LEN];
	char *s = &workString[0];

	if( groupName[0] == '-' )
	{
		return;
	}

	// Ken added because tell.printf() added log entries, and a log callback is calling
	// tell.printf() - so an impasse.  I've added this check for logLock until
	// I resolve the ambiguity - Presumably tell.printf() shouldn't add log entries.
	if( logLock )
		return;

	sysAssert( strlen( fmt ) < (WORKSTRING_LEN*3/4) );

	{
		int indent = 0;
		for( indent=0 ; indent<logIndent ; indent++  ) 
		{
			*(s++) = ' ';
		}
	}

	int n = WORKSTRING_LEN - (s-&workString[0]);
	elipsisTo( ((unsigned int)n), s, fmt );

	{
		LOCK(logLock);
		if( logStream == NULL ) 
		{
			logStream = stdout;
		}
	}

	unsigned char *c = (unsigned char*)workString;
	while( *c )
	{
		if( *(c++) > 127 )
		{
			*c = '\0';
		}
	}


	if( groupName[0] == '!' )
	{
		sysAssertEntryPoint( workString, __FILE__, __LINE__ );
		return;
	}

	sysAssert( groupName[0] == '+' || groupName[0] == '*' );
	// If you're not disallowing the group with '-' then
	// you must have a plus at the beginning signifying that
	// you're printing it.

	{
		int len = strlen( workString );
		int i = len-1;
		int hadOneCr = 0;

		while( i>= 0 && (workString[i]=='\n' || workString[i]=='\r') ) {
			workString[i--] = 0;
			hadOneCr = 1;
		}

		if( hadOneCr ) {
			strcat( workString, "\n" );
		}
	}

	{
		LOCK(logLock);
		if( logCallback )
		{
			logCallback( groupName, workString );
		}
	}

	if( logFileName[0] != '\0' )
	{
		LOCK(logLock);

		//if( logStream && logToStdErr ) {
		//	fprintf( stderr, workString );
		//}


		if( logSuperCareful )
		{	
			logStream = fopen( logFileName, "a+b" );
			if( fileReportOpen )
				fileReportOpen( (int)logStream, logFileName, 0, 1, 1 );
		}

		if( logStream )
		{
			fprintf( logStream, workString );
		}

		if( logFlushAlways )
		{
			fflush( logStream );
		}

		if( logSuperCareful )
		{
			logClose();
		}
	}
}

void logSuppress( int yesNo )
{
	if( yesNo )
	{
		if( !logSuperCareful )
			logClose();	
		logDoNotShow = 1;
	}
	else
	{
		logDoNotShow = 0;
		if( logSuperCareful )
			logOpen( logFileName );	
	}
}


void tell( char *format, ... )
{
	char temp[1024*64];
	elipsisTo( 1024*64, temp, format );
	logAdd( logTell, temp );
}

