#ifndef FULLTIME_H
#define FULLTIME_H

// This is a fully portable time class.  It stores the time
// in double seconds which as much accurarcy possible in the OS
// The time is in seconds since 1 Jan 1997

#define ftINVALID (0.0)

struct FullTime {
	double time;

	FullTime() { time = ftINVALID; }
	FullTime(double f) { time = f; }
	operator double() { return time; }

	// Not all machines use the same floating point encodings
	// so it may sometimes be necessary for portability to
	// convert a full time to fixed point before transmitting
	int getFixedPoint() {
		int t = (int)time;
		return (int((time-(double)t)*2.0*2.0*2.0) | (t<<3));
	}
	
	void setFixedPoint(int t) {
		time = double(t>>3) + double(t&0x7)/8.0;
	}
};

// Global Variables
//------------------------------------------------------------------------

extern int globalFrameNumber;
	// Incremented each time setFrameTime is called 
	// which should be once per frame

extern FullTime frameTime;
	// The time at the start of this frame corrected to server time if appropriate

extern FullTime uncorrectedFrameTime;
	// The time at the start of this frame uncorrected

extern int serverDiffMils;
	// Server difference in mils as derived from time syncronization

extern FullTime lastFrameElapsedTime;
	// Time elapsed on last frame

extern FullTime firstTime;
	// Time that the system was initialized

extern double thisFPS;
	// The last calculated frames per second

#ifndef FRAMES_TO_AVG
	#define FRAMES_TO_AVG 10
#endif
extern double avgFPS;
	// The running average of the last FRAMES_TO_AVG 


// Interface Functions
//------------------------------------------------------------------------

extern FullTime getRealTime();
	// The time at this instant corrected

extern FullTime getUncorrectedTime();
	// The time at this instance uncorrected

void initFullTime();
	// Call this to initialize the system.
	// Under Windows it does some evil calibrartion for up to a second

void setFrameTime();
	// Call this once a frame to set the globals "frameTime" and "uncorrectedFrameTime"
	// The two globals should be used in most time related code
	// to avoid unnecessary calls to kernel timing code

void sleepMils( int mils );
	// Platform independent sleep mils

void stopTime();
	// Stops time and uses an accumulator to collect
	// the "stopped time".  Useful for non-synchronized
	// (i.e. single player games) modes.

void resumeTime();
	// Resumes time

int isTimeStopped();
	// Query time stopped status

void resetStoppedTime();
	// Clears the time stop accumulator.  YOU MUST CALL
	// THIS before you try to use time synchronization!
	// Time sync will assert that the stopTimeAccumulator
	// is 0



// Timer Class
//------------------------------------------------------------------------
// This is a handy timer class that lets you set a stopwatch

struct Timer {
	FullTime timeDone;

	void start(FullTime countDown) { timeDone = frameTime + countDown; }
	void start(float    countDown) { timeDone = frameTime + (double)countDown; }
	void start(double   countDown) { timeDone = frameTime + countDown; }
	void start(int      countDown) { timeDone = frameTime + (double)countDown; }

	Timer() { stop(); }
	Timer(FullTime countDown) { start(countDown); }
	Timer(float    countDown) { start(countDown); }
	Timer(double   countDown) { start(countDown); }
	Timer(int      countDown) { start(countDown); }

	int isDone() { return frameTime >= timeDone; }
	int isRunning() { return timeDone != 0.0; }
	void stop() { timeDone = 0.0; }
};

// Uncorrected timer does not use frameTime, but instead
// queires the clock on each call.
struct UCTimer {
	FullTime timeDone;

	void start(FullTime countDown) { timeDone = getUncorrectedTime() + countDown; }
	void start(float    countDown) { timeDone = getUncorrectedTime() + (double)countDown; }
	void start(double   countDown) { timeDone = getUncorrectedTime() + countDown; }
	void start(int      countDown) { timeDone = getUncorrectedTime() + (double)countDown; }

	UCTimer() { stop(); }
	UCTimer(FullTime countDown) { start(countDown); }
	UCTimer(float    countDown) { start(countDown); }
	UCTimer(double   countDown) { start(countDown); }
	UCTimer(int      countDown) { start(countDown); }

	int isDone() { return getUncorrectedTime() >= timeDone; }
	int isRunning() { return timeDone != 0.0; }
	void stop() { timeDone = 0.0; }
};

#endif
